# /usr/local/src/bibdata/mgdata/mg_get_bibtex.awk, Sun Dec 31 08:29:11 2000
# Edit by Nelson H. F. Beebe <beebe@math.utah.edu>
# /home/0057/src/bibdata/mgdata/mg_get_bibtex.awk, Wed Feb 12 07:14:29 1997
# Edit by Nelson H. F. Beebe <beebe@chamberlin.math.utah.edu>
# ========================================================================
# Filter a BibTeX file for mg, removing comments, @preamble{}, and
# @string{} entries, and creating Ctl-B-separated records that each
# contain a single BibTeX entry, and in each of which, any BibTeX string
# abbreviations used for address, institution, journal, organization,
# publisher, school, and series are also defined by
# @String{name="value"} entries which are output as part of the record
# immediately before the BibTeX entry.
#
# Usage:
#	AWK -f mg_get_bibtex.awk BibTeXfiles(s)
#
# [where AWK == awk, gawk, mawk, or nawk]
#
# [31-Dec-2000] -- remove need for IGNORECASE support, so the much 
#		   faster mawk can be used in place of gawk
#
# NB: gawk is required because it supports the IGNORECASE variable.
#
# [15-Dec-1998] -- improve KEYWORDS output by handling reduction of
# 		   TeX accents and TeX control sequences, and produce
# 		   a list of sorted unique keywords.
# [14-Dec-1998] -- add KEYWORDS output for parts of hyphenated words
# [12-Feb-1997] -- original version
# ========================================================================

BEGIN		{ initialize() }

/^%/		{ next }

/^[ \t]*$/	{ next }

/^@[Cc][Oo][Mm][Mm][Ee][Nn][Tt] *{/	{ do_preamble(); next }

/^@[Pp][Rr][Ee][Aa][Mm][Bb][Ll][Ee] *{/	{ do_preamble(); next }

/^@[Ss][Tt][Rr][Ii][Nn][Gg] *{/		{ do_string(); next }

/^@/					{ do_entry(); next }

/^ *(address|institution|journal|organization|publisher|school|series) *= *[a-zA-Z]/ { do_abbrev(); next }

		{ save($0) }

END		{ print_and_forget_entry(); printf("%c",2) }

# ======================================================================


function brace_level(s, k,level,n)
{
    level = 0
    n = length(s)
    for (k = 1; k <= n; ++k)
    {
	if (substr(s,k,1) == "{")
	    level++
	else if (substr(s,k,1) == "}")
	    level--
    }
    return (level)
}


function deTeXify(s)
{
    ### Debug_s = s

    gsub("{\\\\AA}",		"AA",	s)
    gsub("{\\\\AE}",		"AE",	s)
    gsub("{\\\\L}",		"L",	s)
    gsub("{\\\\OE}",		"OE",	s)
    gsub("{\\\\O}",		"O",	s)
    gsub("{\\\\aa}",		"aa",	s)
    gsub("{\\\\ae}",		"ae",	s)
    gsub("{\\\\l}",		"l",	s)
    gsub("{\\\\oe}",		"oe",	s)
    gsub("{\\\\o}",		"o",	s)
    gsub("{\\\\ss}",		"ss",	s)
    gsub("{\\\\eth}",		"d",	s)
    gsub("{\\\\Eth}",		"D",	s)
    gsub("{\\\\dbar}",		"d",	s)
    gsub("{\\\\Dbar}",		"D",	s)
    gsub("{\\\\thorn}",		"th",	s)
    gsub("{\\\\Thorn}",		"Th",	s)

    gsub("[?]`",		"",	s)
    gsub("!`",			"",	s)
    gsub("{\\\\[`'^\"~=.]",	"",	s)
    gsub("\\\\[bcdHktuv][ {]",	"",	s)

    gsub("\\\\[&]",		" ",	s)	# change ampersand to space

    gsub("\\\\[-,./;:'\"`=_+|~()!@#$%^*0-9]", "", s) # remove all remaining TeX control words
    gsub("\\\\[|~()!@#$%^*0-9]", "",	s)	# remove all remaining TeX control words

    gsub("\\\\(bf|em|it|rm|sc|sf|sl|tt) "," ",s) # remove font changes

    gsub("\\\\cite\\[[^\\]]*\\]{[^}]*}", "", s)	# remove \cite[...]{...}
    gsub("\\\\cite{[^}]*}", "", s)		# remove \cite{...}

    gsub("\\\\[A-Za-z]+",	" & ", s)	# put spaces around all remaining TeX control sequences
    gsub("[$_^]",		" ",	s)	# replace TeX math delimiters by space

    ### print "DEBUG: deTeXify(" Debug_s ") -> (" s ")"

    return (s)
}


function do_abbrev( abbrev)
{
    save($0)
    abbrev = trim(substr($0,index($0,"=")+1))
    sub(/ *,$/,"",abbrev)
    if (abbrev in Define)
	Strings[++NStrings] = abbrev
}


function do_entry()
{
    print_and_forget_entry()
    EntryLineNumber = FNR
    save($0)
}


function do_preamble( level,x)
{
    level = brace_level($0)
    while ((level > 0) && (getline x > 0))
	level += brace_level(x)
}


function do_string( k1,k2,level,name,value,x)
{
    k1 = index($0,"{") + 1
    k2 = index($0,"=")
    name = trim(substr($0,k1,k2-k1))
    value = trim(substr($0,k2+1))
    level = brace_level($0)
    while ((level > 0) && (getline x > 0))
    {
	x = trim(x)
	level += brace_level(x)
	value = value " " x
    }
    gsub(/  +/," ",value)
    Define[name] = value
}


function initialize()
{
    IGNORECASE = 1

    EntryLineNumber = 0
    NStrings = 0
    NLines = 0
}


function less(a,b)
{
    # NB: This function must be updated for each application!
    return (a < b)
}


function partition(array,left,right, i,j,swap,v)
{
    i = left - 1
    j = right
    v = array[right]
    for (;;)
    {
	while (less(array[++i],v))
	    ;
	while (less(v,array[--j]))
	{
	    if (j == left)
		break
	}
	if (i >= j)
	    break
	swap = array[i]
	array[i] = array[j]
	array[j] = swap
    }
    swap = array[i]
    array[i] = array[right]
    array[right] = swap
    return (i)
}


function print_and_forget_entry( href,is_accented_word,k,keywords,last_word,\
				 m,n,the_line,the_word,unique_extra_words,words)
{
    if (NLines > 0)
    {
	href = FILENAME
	sub(/^\/u\/ftp/, "ftp://ftp.math.utah.edu", href)
	printf("%cURL = %s Line=%d\n\n", 2, href, EntryLineNumber)
    }

    # Split compound (hyphenated and/or slashed) words, and output them
    # in a separate KEYWORDS list, because mg does not otherwise index
    # their parts.  We also include accented words, with accents removed,
    # since it is not otherwise obvious to a user how to search reliably
    # for such a word.  However, we exclude all one-letter words, so that
    # accented initials are not indexed.
    #
    # We first construct a list of unique lowercased compound words that
    # contain only letters, hyphens, and slashes, and begin and end with
    # a letter, or else, had embedded accents.  To save time, values for
    # several keys unlikely to have any compound words that are useful
    # to index are skipped.
    for (k = 1; k <= NLines; ++k)
    {
	the_line = Lines[k]
	if (the_line ~ /^[ \t]*(acknowledgement|bibdate|bibsource|classcodes|classification|CODEN|day|govtdocnumber|ISBN|ISSN|LCCN|month|MRclass|MRnumber|number|pages|price|URL|USMARC.*|volume|year)[ \t]*=/)
	    continue

	gsub("[\t]",		" ",	the_line) # change tabs to spaces
	gsub("\\\\slash",	" ",	the_line) # reduce slash to space
	gsub("\\\\emdash",	" ",	the_line) # reduce emdash to space
	gsub("\\\\-",		"",	the_line) # remove discretionary hyphens
	gsub("\\\\ ",		"",	the_line) # remove fixed space

	n = split(the_line,words," ")
	for (m = 1; m <= n; ++m)
	{
	    the_word = words[m]
	    is_accented_word = (the_word ~ "[\\\\`^_$]") # match control sequence/word, Spanish ligature (?`, !`), math mode
	    if (is_accented_word)		# this simple test avoids expensive accent removal
		the_word = deTeXify(the_word)
	    gsub("'s[}]? *$","",the_word)		# remove possessives
	    gsub("[{}\"`']","",the_word)	# remove all remaining braces and quotation marks
	    gsub("^[([]+","",the_word)		# remove outer parentheses and brackets and
	    gsub("[])]*[.,;:/]*$","",the_word)	# any trailing punctuation
	    gsub("  +"," ",the_word)		# collapse multiple spaces to single spaces
	    the_word = trim(the_word)
	    if ((length(the_word) > 1) && (is_accented_word || \
		((the_word ~ "^[A-Za-z]+[-/][A-Za-z]+([-/][A-Za-z]+)*$") && \
		(the_word !~ "^(ack|inst|j|pub|ser)-"))))
		unique_extra_words[tolower(the_word)]++
	}
    }
    delete unique_extra_words["i/o"]		# this word is too common

    keywords = ""
    for (the_word in unique_extra_words)
	keywords = keywords " " the_word
    gsub("[-/]"," ",keywords)
    keywords = trim(keywords)

    # Our reductions may have introduced additional spaces into words,
    # and there may be duplicates (e.g., from Gauss-Legendre and
    # Gauss-Laguerre), so resplit, sort, and rebuild the word list with
    # duplicates and one-letter words eliminated.
    n = split(keywords,words," ")
    quicksort(words,1,n)

    # Construct a string containing the sorted compound words, with
    # hyphens and slashes changed to spaces, and if it is not empty,
    # output it.
    keywords = ""
    last_word = ""
    for (k = 1; k <= n; ++k)
    {
	the_word = words[k]
	if ((length(the_word) > 1) && (last_word != the_word))
	    keywords = keywords " " the_word
	last_word = the_word
    }
    keywords = trim(keywords)

    if (keywords != "")
	printf("KEYWORDS = %s\n\n", keywords)

    for (k = 1; k <= NStrings; ++k)
	printf("@String{%-23s = %s\n\n", Strings[k], Define[Strings[k]])
    NStrings = 0

    # Finally, output the complete bibliographic entry:
    for (k = 1; k <= NLines; ++k)
    {
	the_line = Lines[k]
	gsub(/\\emdash/,"{}\\emdash",the_line)
	print the_line
    }
    print ""	# follow with a blank line so entry is a paragraph in bibsearch output
    NLines = 0
}


function quicksort(array,left,right, i)
{
    # The code in partition() and quicksort() is a direct translation
    # of the simple quicksort algorithm given in Robert Sedgewick's
    # ``Algorithms in C'', 3rd edition, Addison-Wesley, 1998,
    # pp. 305--307.  We need an O(N lg N) algorithm here instead of a
    # simpler O(N^2) algorithm because the font list has thousands of
    # entries.  There are many things that one can do to tweak
    # quicksort() to make its worst-case behavior of O(N^2) unlikely,
    # and to improve its performance on small sequences by switching
    # to other sorting algorithms.  However, we do not attempt any of
    # those refinements here.
    #
    # The user-defined less(a,b) function conceals the details of how
    # array items are compared.

    if (right <= left)
	return
    i = partition(array,left,right)
    quicksort(array, left, i - 1)
    quicksort(array, i + 1, right)
}


function save(s)
{
    Lines[++NLines] = s
}


function trim(s)
{
    sub(/^[ \t]+/,"",s)
    sub(/[ \t]+$/,"",s)
    return (s)
}
